@@ -7,8 +7,8 @@ Provides IPython console widget.
 @copyright: Copyright (c) 2007 IBM Corporation
 @license: BSD
 
-All rights reserved. This program and the accompanying materials are made 
-available under the terms of the BSD which accompanies this distribution, and 
+All rights reserved. This program and the accompanying materials are made
+available under the terms of the BSD which accompanies this distribution, and
 is available at U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 
@@ -36,7 +36,7 @@ except ImportError:
 class IterableIPShell:
   '''
   Create an IPython instance. Does not start a blocking event loop,
-  instead allow single iterations. This allows embedding in GTK+ 
+  instead allow single iterations. This allows embedding in GTK+
   without blockage.
 
   @ivar IP: IPython instance.
@@ -44,17 +44,17 @@ class IterableIPShell:
   @ivar iter_more: Indicates if the line executed was a complete command,
   or we should wait for more.
   @type iter_more: integer
-  @ivar history_level: The place in history where we currently are 
+  @ivar history_level: The place in history where we currently are
   when pressing up/down.
   @type history_level: integer
   @ivar complete_sep: Seperation delimeters for completion function.
   @type complete_sep: _sre.SRE_Pattern
   '''
-  def __init__(self,argv=[],user_ns=None,user_global_ns=None, 
+  def __init__(self,argv=[],user_ns=None,user_global_ns=None,
                cin=None, cout=None,cerr=None, input_func=None):
     '''
-    
-    
+
+
     @param argv: Command line options for IPython
     @type argv: list
     @param user_ns: User namespace.
@@ -64,7 +64,7 @@ class IterableIPShell:
     @param cin: Console standard input.
     @type cin: IO stream
     @param cout: Console standard output.
-    @type cout: IO stream 
+    @type cout: IO stream
     @param cerr: Console standard error.
     @type cerr: IO stream
     @param input_func: Replacement for builtin raw_input()
@@ -91,13 +91,13 @@ class IterableIPShell:
       if cerr:
         sys.stderr = cerr
 
-    # This is to get rid of the blockage that accurs during 
+    # This is to get rid of the blockage that accurs during
     # IPython.Shell.InteractiveShell.user_setup()
 
     io.raw_input = lambda x: None
 
     os.environ['TERM'] = 'dumb'
-    excepthook = sys.excepthook 
+    excepthook = sys.excepthook
 
     if parse_version(IPython.release.version) >= parse_version('4.0.0'):
         from traitlets.config.loader import Config
@@ -135,7 +135,7 @@ class IterableIPShell:
     sys.excepthook = excepthook
     self.iter_more = 0
     self.history_level = 0
-    self.complete_sep =  re.compile('[\s\{\}\[\]\(\)]')
+    self.complete_sep = re.compile(r'[\s\{\}\[\]\(\)]')
     self.updateNamespace({'exit':lambda:None})
     self.updateNamespace({'quit':lambda:None})
     if parse_version(IPython.release.version) < parse_version("5.0.0"):
@@ -204,7 +204,6 @@ class IterableIPShell:
       else:
         self.IP.input_splitter.push(line)
         self.iter_more = self.IP.input_splitter.push_accepts_more()
-      self.prompt = self.generatePrompt(self.iter_more)
       if not self.iter_more:
           if self.no_input_splitter:
             source_raw = '\n'.join(self.lines)
@@ -220,6 +219,7 @@ class IterableIPShell:
           #
           self.IP.rl_do_indent = True
           pass
+      self.prompt = self.generatePrompt(self.iter_more)
 
     if IPython.version_info < (8,):
       sys.stdout = orig_stdout
@@ -230,7 +230,7 @@ class IterableIPShell:
     Generate prompt depending on is_continuation value
 
     @param is_continuation
-    @type is_continuation: boolean 
+    @type is_continuation: boolean
 
     @return: The prompt string representation
     @rtype: string
@@ -257,7 +257,7 @@ class IterableIPShell:
   def historyBack(self):
     '''
     Provides one history command back.
-    
+
     @return: The command string.
     @rtype: string
     '''
@@ -265,22 +265,22 @@ class IterableIPShell:
     if not self._getHistory():
       self.history_level +=1
     return self._getHistory()
-  
+
   def historyForward(self):
     '''
     Provides one history command forward.
-    
+
     @return: The command string.
     @rtype: string
     '''
     if self.history_level < 0:
       self.history_level += 1
     return self._getHistory()
-  
+
   def _getHistory(self):
     '''
     Get's the command string of the current history level.
-    
+
     @return: Historic command string.
     @rtype: string
     '''
@@ -293,7 +293,7 @@ class IterableIPShell:
   def updateNamespace(self, ns_dict):
     '''
     Add the current dictionary to the shell namespace.
-    
+
     @param ns_dict: A dictionary of symbol-values.
     @type ns_dict: dictionary
     '''
@@ -302,11 +302,11 @@ class IterableIPShell:
   def complete(self, line):
     '''
     Returns an auto completed line and/or posibilities for completion.
-    
+
     @param line: Given line so far.
     @type line: string
-    
-    @return: Line completed as for as possible, 
+
+    @return: Line completed as for as possible,
     and possible further completions.
     @rtype: tuple
     '''
@@ -320,12 +320,12 @@ class IterableIPShell:
       def _commonPrefix(str1, str2):
         '''
         Reduction function. returns common prefix of two given strings.
-        
+
         @param str1: First string.
         @type str1: string
         @param str2: Second string
         @type str2: string
-        
+
         @return: Common prefix to both strings.
         @rtype: string
         '''
@@ -335,18 +335,23 @@ class IterableIPShell:
         return str1
       if possibilities[1]:
         common_prefix = reduce(_commonPrefix, possibilities[1]) or split_line[-1]
-        completed = line[:-len(split_line[-1])]+common_prefix
+        autocomplete_start_index = line.rfind(possibilities[0])
+        completed = line[0:autocomplete_start_index] + common_prefix
+        # suggestions for current line consist of text not used for completion + completion matches
+        unmatched_text = line[-len(split_line[-1]):autocomplete_start_index]
+        suggestions = [unmatched_text + p for p in possibilities[1]]
+        possibilities = (possibilities[0], suggestions)
       else:
         completed = line
     else:
       completed = line
     return completed, possibilities[1]
-  
+
 
   def shell(self, cmd,verbose=0,debug=0,header=''):
     '''
     Replacement method to allow shell commands without them blocking.
-    
+
     @param cmd: Shell command to execute.
     @type cmd: string
     @param verbose: Verbosity
@@ -369,7 +374,8 @@ class ConsoleView(gtk.TextView):
   '''
   Specialized text view for console-like workflow.
 
-  @cvar ANSI_COLORS: Mapping of terminal colors to X11 names.
+  @cvar ANSI_COLORS: Mapping of terminal control sequence values to
+                     tuples containing foreground and background color names.
   @type ANSI_COLORS: dictionary
 
   @ivar text_buffer: Widget's text buffer.
@@ -381,16 +387,38 @@ class ConsoleView(gtk.TextView):
   @ivar line_start: Start of command line mark.
   @type line_start: gtk.TextMark
   '''
-  ANSI_COLORS =  {'0;30': 'Black',     '0;31': 'Red',
-                  '0;32': 'Green',     '0;33': 'Brown',
-                  '0;34': 'Blue',      '0;35': 'Purple',
-                  '0;36': 'Cyan',      '0;37': 'LightGray',
-                  '1;30': 'DarkGray',  '1;31': 'DarkRed',
-                  '1;32': 'SeaGreen',  '1;33': 'Yellow',
-                  '1;34': 'LightBlue', '1;35': 'MediumPurple',
-                  '1;36': 'LightCyan', '1;37': 'White',
-                  '38;5;241': 'Gray',
-                  '39': 'Black' }
+  ANSI_COLORS = {'0;30': ('Black', None),
+                 '0;31': ('Red', None),
+                 '0;32': ('Green', None),
+                 '0;33': ('Brown', None),
+                 '0;34': ('Blue', None),
+                 '0;35': ('Purple', None),
+                 '0;36': ('Cyan', None),
+                 '0;37': ('LightGray', None),
+                 '1;30': ('DarkGray', None),
+                 '1;31': ('DarkRed', None),
+                 '1;32': ('SeaGreen', None),
+                 '1;33': ('Yellow', None),
+                 '1;34': ('LightBlue', None),
+                 '1;35': ('MediumPurple', None),
+                 '1;36': ('LightCyan', None),
+                 '1;37': ('White', None),
+                 '38;5;124;03': ('DarkRed', None),
+                 '38;5;124;43': ('DarkRed', 'Yellow'),
+                 '38;5;21': ('Blue', None),
+                 '38;5;241': ('Gray', None),
+                 '38;5;241;43': ('Gray', 'Yellow'),
+                 '38;5;250': ('Silver', None),
+                 '38;5;28': ('Green', None),
+                 '38;5;28;01': ('Green', None),
+                 '38;5;28;43': ('Green', 'Yellow'),
+                 '38;5;28;43;01': ('Green', 'Yellow'),
+                 '39': ('Black', None),
+                 '39;00': ('Black', None),
+                 '39;49': ('Red', 'White'),
+                 '39;49;00': ('Red', 'White'),
+                 '43': (None, 'Yellow'),
+                 '49': (None, 'White')}
 
   def __init__(self):
     '''
@@ -413,28 +441,29 @@ class ConsoleView(gtk.TextView):
     self.modify_font(Pango.FontDescription(chosen))
     self.set_cursor_visible(True)
     self.text_buffer = self.get_buffer()
-    self.mark = self.text_buffer.create_mark('scroll_mark', 
+    self.mark = self.text_buffer.create_mark('scroll_mark',
                                              self.text_buffer.get_end_iter(),
                                              False)
     for code in self.ANSI_COLORS:
-      self.text_buffer.create_tag(code, 
-                                  foreground=self.ANSI_COLORS[code], 
+      self.text_buffer.create_tag(code,
+                                  foreground=self.ANSI_COLORS[code][0],
+                                  background=self.ANSI_COLORS[code][1],
                                   weight=700)
     self.text_buffer.create_tag('0')
     self.text_buffer.create_tag('notouch', editable=False)
-    self.color_pat = re.compile('\x01?\x1b\[(.*?)m\x02?')
+    self.color_pat = re.compile(r'\x01?\x1b\[(.*?)m\x02?')
     self.line_start = \
-        self.text_buffer.create_mark('line_start', 
+        self.text_buffer.create_mark('line_start',
                                      self.text_buffer.get_end_iter(), True)
     self.connect('key-press-event', self.onKeyPress)
-    
+
   def write(self, text, editable=False):
     GLib.idle_add(self._write, text, editable)
 
   def _write(self, text, editable=False):
     '''
     Write given text to buffer.
-    
+
     @param text: Text to append.
     @type text: string
     @param editable: If true, added text is editable.
@@ -442,8 +471,8 @@ class ConsoleView(gtk.TextView):
     '''
     segments = self.color_pat.split(text)
     segment = segments.pop(0)
-    start_mark = self.text_buffer.create_mark(None, 
-                                              self.text_buffer.get_end_iter(), 
+    start_mark = self.text_buffer.create_mark(None,
+                                              self.text_buffer.get_end_iter(),
                                               True)
     self.text_buffer.insert(self.text_buffer.get_end_iter(), segment)
 
@@ -468,7 +497,7 @@ class ConsoleView(gtk.TextView):
   def _showPrompt(self, prompt):
     '''
     Prints prompt at start of line.
-    
+
     @param prompt: Prompt to print.
     @type prompt: string
     '''
@@ -482,7 +511,7 @@ class ConsoleView(gtk.TextView):
   def _changeLine(self, text):
     '''
     Replace currently entered command line with given text.
-    
+
     @param text: Text to use as replacement.
     @type text: string
     '''
@@ -494,7 +523,7 @@ class ConsoleView(gtk.TextView):
   def getCurrentLine(self):
     '''
     Get text in current command line.
-    
+
     @return: Text of current command line.
     @rtype: string
     '''
@@ -509,14 +538,14 @@ class ConsoleView(gtk.TextView):
   def _showReturned(self, text):
     '''
     Show returned text from last command and print new prompt.
-    
+
     @param text: Text to show.
     @type text: string
     '''
     iter = self.text_buffer.get_iter_at_mark(self.line_start)
     iter.forward_to_line_end()
     self.text_buffer.apply_tag_by_name(
-      'notouch', 
+      'notouch',
       self.text_buffer.get_iter_at_mark(self.line_start),
       iter)
     self._write('\n'+text)
@@ -535,15 +564,15 @@ class ConsoleView(gtk.TextView):
 
   def onKeyPress(self, widget, event):
     '''
-    Key press callback used for correcting behavior for console-like 
+    Key press callback used for correcting behavior for console-like
     interfaces. For example 'home' should go to prompt, not to begining of
     line.
-    
+
     @param widget: Widget that key press accored in.
     @type widget: gtk.Widget
     @param event: Event object
     @type event: gtk.gdk.Event
-    
+
     @return: Return True if event should not trickle.
     @rtype: boolean
     '''
@@ -575,6 +604,15 @@ class ConsoleView(gtk.TextView):
         self.text_buffer.move_mark(self.line_start, self.text_buffer.get_iter_at_offset(len(self.prompt)))
         self.text_buffer.place_cursor(self.text_buffer.get_iter_at_offset(cursor_pos_in_line))
         return True
+    elif event.state & gdk.ModifierType.CONTROL_MASK and event.keyval in [gdk.KEY_k, gdk.KEY_K]:
+      # clear text after input cursor on Ctrl+K
+      if insert_iter.editable(True):
+        self.text_buffer.delete(insert_iter, self.text_buffer.get_end_iter())
+      return True
+    elif event.state & gdk.ModifierType.CONTROL_MASK and event.keyval == gdk.KEY_C:
+      # copy selection on Ctrl+C (upper-case 'C' only)
+      self.text_buffer.copy_clipboard(gtk.Clipboard.get(gdk.SELECTION_CLIPBOARD))
+      return True
     elif not event.string:
       pass
     elif start_iter.compare(insert_iter) <= 0 and \
@@ -586,7 +624,7 @@ class ConsoleView(gtk.TextView):
     elif insert_iter.compare(selection_iter) < 0:
       self.text_buffer.move_mark(insert_mark, start_iter)
     elif insert_iter.compare(selection_iter) > 0:
-      self.text_buffer.move_mark(selection_mark, start_iter)             
+      self.text_buffer.move_mark(selection_mark, start_iter)
 
     return self.onKeyPressExtend(event)
 
@@ -619,10 +657,10 @@ class IPythonView(ConsoleView, IterableIPShell):
   def raw_input(self, prompt=''):
     '''
     Custom raw_input() replacement. Get's current line from console buffer.
-    
+
     @param prompt: Prompt to print. Here for compatability as replacement.
     @type prompt: string
-    
+
     @return: The current command line text.
     @rtype: string
     '''
@@ -633,18 +671,18 @@ class IPythonView(ConsoleView, IterableIPShell):
 
   def onKeyPressExtend(self, event):
     '''
-    Key press callback with plenty of shell goodness, like history, 
+    Key press callback with plenty of shell goodness, like history,
     autocompletions, etc.
-    
+
     @param widget: Widget that key press occured in.
     @type widget: gtk.Widget
     @param event: Event object.
     @type event: gtk.gdk.Event
-    
+
     @return: True if event should not trickle.
     @rtype: boolean
     '''
-    if event.state & gdk.ModifierType.CONTROL_MASK and event.keyval == 99:
+    if event.state & gdk.ModifierType.CONTROL_MASK and event.keyval == gdk.KEY_c:
       self.interrupt = True
       self._processLine()
       return True
@@ -681,7 +719,7 @@ class IPythonView(ConsoleView, IterableIPShell):
     self.showReturned(rv)
     self.cout.truncate(0)
     self.cout.seek(0)
- 
+
 if __name__ == "__main__":
   window = gtk.Window()
   window.set_default_size(640, 320)
@@ -689,4 +727,4 @@ if __name__ == "__main__":
   window.add(IPythonView())
   window.show_all()
   gtk.main()
- 
+
